#include "MemoryMng.h"
#include <string.h>

CMemoryMng g_MemMng;

CMemoryMng::CMemoryMng(){
#ifdef WIN32
	hProcess = OpenProcess(PROCESS_ALL_ACCESS,FALSE, GetCurrentProcessId());
#endif
}

CMemoryMng::~CMemoryMng(){
#ifdef WIN32
	CloseHandle(hProcess);
#endif
}

bool CMemoryMng::WriteMemmory(const void* lpBaseAddress, const void* lpBuffer, const int nSize){
	bool bRc;
#ifdef WIN32
	bRc = WriteProcessMemory(hProcess, (LPVOID)lpBaseAddress, lpBuffer, nSize, NULL) == TRUE;
#else
	unprotect(lpBaseAddress, nSize);

	unsigned int p_size = sysconf(_SC_PAGESIZE);
	char* all_adr = (char*)(((unsigned int)lpBaseAddress) & ~(p_size-1));
	unsigned int size = (unsigned int)lpBaseAddress - (unsigned int)all_adr + nSize;

	mlock(all_adr, size);
	memcpy((void*)lpBaseAddress, lpBuffer, nSize);
	munlock(all_adr, size);
	bRc = true;
#endif
	return bRc;
}
bool CMemoryMng::ReadMemmory(const void* lpBaseAddress, const void* lpBuffer, const int nSize){
	bool bRc;
#ifdef WIN32
	bRc = ReadProcessMemory(hProcess, lpBaseAddress, (LPVOID)lpBuffer, nSize, NULL) == TRUE;
#else

	unprotect(lpBaseAddress, nSize);

	unsigned int p_size = sysconf(_SC_PAGESIZE);
	char* all_adr = (char*)(((unsigned int)lpBaseAddress) & ~(p_size-1));
	unsigned int size = (unsigned int)lpBaseAddress - (unsigned int)all_adr + nSize;

	mlock(all_adr, size);
	memcpy((void*)lpBuffer, lpBaseAddress,nSize);
	munlock(all_adr, size);
	bRc = true;
#endif
	return bRc;
}

bool CMemoryMng::unprotect(const void* base_adr, int len){
	bool bRc = false;
#ifdef WIN32
	DWORD oldprv;
	if( VirtualProtect((void*)base_adr, len, PAGE_EXECUTE_READWRITE,&oldprv) != 0)
		bRc = true;
#else
	unsigned int p_size = sysconf(_SC_PAGESIZE);
	char* all_adr = (char*)(((unsigned int)base_adr) & ~(p_size-1));
	unsigned int size = (unsigned int)base_adr - (unsigned int)all_adr + len;
	if( mprotect(all_adr, size, PROT_READ|PROT_WRITE|PROT_EXEC) == 0)
		bRc = true;
#endif
	return bRc;
}